// Copyright Earl Warren <contact@earl-warren.org>
// Copyright Loïc Dachary <loic@dachary.org>
// SPDX-License-Identifier: MIT

package generic

import (
	"context"

	"code.forgejo.org/f3/gof3/v3/logger"
	"code.forgejo.org/f3/gof3/v3/options"
)

type TreeInterface interface {
	logger.Interface

	Init(TreeInterface, options.Interface) TreeInterface

	GetOptions() options.Interface
	SetOptions(options.Interface)

	GetSelf() TreeInterface
	SetSelf(TreeInterface)

	SetRoot(NodeInterface)
	GetRoot() NodeInterface

	SetLogger(logger.Interface)
	GetLogger() logger.Interface

	GetDriver() TreeDriverInterface
	SetDriver(TreeDriverInterface)

	GetChildrenKind(Kind) Kind

	GetPageSize() int

	AllocateID() bool

	Clear(context.Context)

	Diff(a, b NodeInterface) string

	MustFind(Path) NodeInterface
	Find(Path) NodeInterface
	FindAndGet(context.Context, Path) NodeInterface
	Exists(context.Context, Path) bool

	Walk(context.Context, *WalkOptions)
	WalkAndGet(context.Context, *WalkOptions)
	Apply(context.Context, Path, *ApplyOptions) bool
	ApplyAndGet(context.Context, Path, *ApplyOptions) bool

	Register(kind Kind, factory FactoryFun)
	Factory(ctx context.Context, kind Kind) NodeInterface
}
